/*
Whenis -- Display dates in various formats

    Copyright (C) 2003  Norm London, London Computing

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	norm_london@londoncomputing.com

    Compile with Visual Studio 6 or later or gcc 2.95.02 or later.  
    cl whenis.c -or- gcc whenis.c

	Usage is: whenis date fmt where date is a date and fmt gives the format
	slash stands for either a /, . or :
	fmt = ymd for yyyymmdd or yyyy/mm/dd
	fmt = mdy for mmddyyyy or mm/dd/yyyy
	fmt = dmy for ddmmyyyy or dd/mm/yyyy
	fmt = sec for seconds from 1970/01/01 
	fmt = jul for yyyyddd (year followed by day of year)
	format is required.

	This source is compatible with both Windows and Unix
*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#ifdef WIN32
#include <process.h>
#endif
#include <ctype.h>
int days_in_month[12] = {31,28,31,30,31,30,31,31,30,31,30,31};
char WeekDay[][9] =
    {"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"};
char timedte[255], fmt;
time_t tim;
int i;
struct tm *tm;
long time_offset;
struct tm timeptr;
int yr,mo,day,julday;
char gmtimestr[64];

char *ascgmtime(struct tm *tm) {
	static char WeekDay[][4] =
    {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	static char Month[][4] = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
	char daystr[4], hourstr[4], minstr[4], secstr[4];

	sprintf(daystr,"%3d",tm->tm_mday+100);
	sprintf(hourstr,"%3d",tm->tm_hour+100);
	sprintf(minstr,"%3d",tm->tm_min+100);
	sprintf(secstr,"%3d",tm->tm_sec+100);
	sprintf(gmtimestr,"%3s %3s %2s %2s:%2s:%2s %4d\n",
		WeekDay[tm->tm_wday],Month[tm->tm_mon],daystr+1,hourstr+1,minstr+1,secstr+1,tm->tm_year+1900);
	return gmtimestr;
}

int day_of_week(int yr,int julday) {
	int year = yr - 1;
	/* julday starts with 0 */
	long days;
	days = (year * 365l) + (year / 4) - (year / 100) + (year / 400) + julday;
	return (days+1) % 7;
}

set_hhmmss(char *hhmmss) {
	if (hhmmss[2] != ':') {
		fprintf(stderr,"%s -- (%s) Invalid Time.\n",timedte,hhmmss);
		exit(4);
	}
	else 
		hhmmss[2] = 0;

	if (hhmmss[5] != ':') {
		fprintf(stderr,"%s -- (%s) Invalid Time.\n",timedte,hhmmss+3);
		exit(4);
	}
	else
		hhmmss[5] = 0;

	timeptr.tm_hour = atoi(hhmmss);
	if (timeptr.tm_hour < 0 || timeptr.tm_hour > 23) {
		fprintf(stderr,"%s -- Invalid Time.\n",timedte,hhmmss);
		exit(4);
	}
	timeptr.tm_min = atoi(hhmmss+3);
	if (timeptr.tm_min < 0 || timeptr.tm_min > 59) {
		fprintf(stderr,"%s -- (%s) Invalid Time.\n",timedte,hhmmss);
		exit(4);
	}

	timeptr.tm_sec = atoi(hhmmss+6);
	if (timeptr.tm_sec < 0 || timeptr.tm_sec > 59) {
		fprintf(stderr,"%s -- (%s) Invalid Time.\n",timedte,hhmmss);
		exit(4);
	}

}


calculate_yearday_seconds() {
	int year;
	// calculate informatin from month,day,year or yearday, year
	timeptr.tm_year -= 1900;
	tim = mktime(&timeptr);
	if (tim== -1) {
		timeptr.tm_year += 1900; // Undo subtract
		// mktime can't deal with this, we'll do it ourselves
		// have mdy
		if (timeptr.tm_year % 4 == 0 && (
			timeptr.tm_year % 100 != 0 || timeptr.tm_year % 400 == 0))
			days_in_month[1]++; /* leap year */
			tim = -1;
		tm = &timeptr;
		timeptr.tm_yday = 0;
		for (i=0;i<timeptr.tm_mon;++i)
			timeptr.tm_yday += days_in_month[i];
		timeptr.tm_yday += timeptr.tm_mday - 1;
		tm->tm_wday = day_of_week(tm->tm_year, tm->tm_yday);
		timeptr.tm_year -= 1900;
	}
	else {
		tm = localtime(&tim);
	}
}

void main(int argc, char *argv[]) {
	int year, month, day, yearday;
	double timesecs;

	if (argc > 1)
		strcpy(timedte,argv[1]);
	else
		timedte[0] = 0;

	if (argc > 2) 
		fmt = toupper(argv[2][0]);
	else
		fmt = ' ';

	timeptr.tm_sec=0;
	timeptr.tm_min=0;
	timeptr.tm_hour=0;
	timeptr.tm_mday=0;
	timeptr.tm_mon=0;
	timeptr.tm_year=0;
	timeptr.tm_wday=0;
	timeptr.tm_yday=0;
	timeptr.tm_isdst=0;

	// For purposes of this conversions, always assume GMT
//#ifdef WIN32
//	if (argc > 2) {
//		if( _putenv( "TZ=GMT" ) == -1 )    {
//			fprintf(stderr, "Unable to set TZ\n" );
//			exit( 1 );
//		}
//		else {
//			_tzset();
 //		}
//	}
//#endif



	if (argc < 2) {
	    fprintf(stderr,"WHENIS Version 1.00, %s\n",__DATE__);
  	    fprintf(stderr,"\nCopyright 2003, Norm London, London Computing.\nnorm_london@londoncomputing.com\n\n");
        fprintf(stderr,"This program is free software; you can redistribute it and/or modify\n");
		fprintf(stderr,"it under the terms of the GNU General Public License as published by\n");
		fprintf(stderr,"the Free Software Foundation; either version 2 of the License, or\n");
		fprintf(stderr,"(at your option) any later version.\n\n");
		fprintf(stderr,"This program is distributed in the hope that it will be useful,\n");
		fprintf(stderr,"but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
		fprintf(stderr,"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
		fprintf(stderr,"GNU General Public License for more details.\n\n");
		fprintf(stderr,"You should have received a copy of the GNU General Public License\n");
		fprintf(stderr,"along with this program; if not, write to the Free Software\n");
		fprintf(stderr,"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\n");
		fprintf(stderr,"Press the Enter key to continue");
		getchar();
		fprintf(stderr,"\nUsage is: WHENIS [date[:hh:mm:ss] fmt]\n");
	    fprintf(stderr,"Date is the date to convert based on fmt.\n");
	    fprintf(stderr,"If no argument specified, todays date is used.\n");
		fprintf(stderr,"Displays date as a standard C date, e.g. Fri Jan 22 18:05:36 1999, \n");
		fprintf(stderr,"seconds since 1/1/1970, in mm/dd/yyy format and as the year and day of year.\n");
		fprintf(stderr,"fmt = ymd for yyyymmdd or yyyy/mm/dd.  fmt = mdy for mmddyyyy or mm/dd/yyyy. \n");
		fprintf(stderr,"fmt = dmy for ddmmyyyy or dd/mm/yyyy.  fmt = sec for seconds from 01/01/1970.\n");
		fprintf(stderr,"fmt = jul for yyyyddd[:hh:mm:ss].      ddd = day of year.\n");
		fprintf(stderr,"All times and dates are relative to local times. \n\n");
}

	switch (fmt) {
	case 'S': 
		timesecs = atof(timedte);
		if (timesecs > 0x7fffffff) {
			fprintf(stderr,"Seconds must be between 0 and %d.\n",0x7fffffff);
			exit(4);
		}
		tim = timesecs;
		if (tim == -1)
			tm = NULL;
		else
			tm = localtime(&tim);
		break;
	case ' ':
		tim = time(&tim);
		if (tim == -1)
			tm = NULL;
		else
			tm = localtime(&tim);
		break;
	case 'J': /* format is yyyyddd */
		if (timedte[7] == ':') {
			set_hhmmss(timedte+8);
			timedte[7]=0;
		}
		julday = atoi(timedte+4);
		timedte[4] = 0;
		timeptr.tm_year = atoi(timedte);
		if (timeptr.tm_year < 1980 || timeptr.tm_year > 2038) {
		    if (timeptr.tm_year % 4 == 0 && (
			timeptr.tm_year % 100 != 0 || timeptr.tm_year % 400 == 0))
				days_in_month[1]++; /* leap year */
		    tim = -1;
		    tm = &timeptr;
		    tm->tm_yday = julday - 1;
		    for (timeptr.tm_mon=0;
			julday > days_in_month[timeptr.tm_mon];
			julday -= days_in_month[timeptr.tm_mon], ++timeptr.tm_mon);
		    tm->tm_mday = julday;
		    tm->tm_wday = day_of_week(tm->tm_year, tm->tm_yday);
			timeptr.tm_year -= 1900;
		}
		else {
			timeptr.tm_year -= 1900;
		    tim = mktime(&timeptr);
			if (tim == -1)
				tm = NULL;
			else {
				tim +=  (julday * (24*3600l));
				tm = localtime(&tim);
			}
		}

		break;

	case 'Y':
		// Assume it is yyyymmdd format
		if (timedte[8] == ':') {
			set_hhmmss(timedte+9);
			timedte[8]=0;
		}
		if (timedte[10]== ':') {
			set_hhmmss(timedte+11);
			timedte[10] = 0;
		}
		strcpy(timedte,argv[1]);
		if (isdigit(timedte[4])) {
			timeptr.tm_mday = atoi(timedte+6);
			timedte[6] = 0;
			timeptr.tm_mon = atoi(timedte+4) - 1;
			timedte[4] = 0;
			timeptr.tm_year = atoi(timedte);
		}
		else {
			// format is yyyy/mm/dd.
			timedte[4] = 0;
			timedte[7] = 0;
			timeptr.tm_mday = atoi(timedte+8);
			timeptr.tm_mon = atoi(timedte+5) - 1;
			timeptr.tm_year = atoi(timedte);
		}
		calculate_yearday_seconds();
		break;
	
	
	case 'M': /* format is mmddyyyy or mm/dd/yy */
		if (timedte[8] == ':') {
			set_hhmmss(timedte+9);
			timedte[8]=0;
		}
		if (timedte[10]== ':') {
			set_hhmmss(timedte+11);
			timedte[10] = 0;
		}
		
		if (isdigit(timedte[2])) {
			// format is mmddyyyy
			timeptr.tm_year = atoi(timedte+4);
			timedte[4] = 0;
			timeptr.tm_mday = atoi(timedte+2);
			timedte[2] = 0;
			timeptr.tm_mon = atoi(timedte) - 1;
		}
		else {
			timedte[2] = 0;
			timedte[5] = 0;
			timeptr.tm_mon = atoi(timedte) - 1;
			timeptr.tm_mday= atoi(timedte+3);
			timeptr.tm_year= atoi(timedte+6);
		}
		calculate_yearday_seconds();
		break;
	
	case 'D': /* format is ddmmyyyy or dd/mm/yyyy */
		if (timedte[8] == ':') {
			set_hhmmss(timedte+9);
			timedte[8]=0;
		}
		if (timedte[10]== ':') {
			set_hhmmss(timedte+11);
			timedte[10] = 0;
		}
		
		if (isdigit(timedte[2])) {
			// format is mmddyyyy
			timeptr.tm_year = atoi(timedte+4);
			timedte[4] = 0;
			timeptr.tm_mday = atoi(timedte+2);
			timedte[2] = 0;
			timeptr.tm_mon = atoi(timedte) - 1;
		}
		else {
			timedte[2] = 0;
			timedte[5] = 0;
			timeptr.tm_mon = atoi(timedte+3) - 1;
			timeptr.tm_mday= atoi(timedte);
			timeptr.tm_year= atoi(timedte+6);
		}
		calculate_yearday_seconds();
		break;

	default:
		tm = NULL;
		break;
	}

	if (tm == NULL) {
	    fprintf(stderr,"%s <=== Bad Date\n",argv[1]);
	    exit(4);
	}

	year = tm->tm_year;
	month = tm->tm_mon+1;
	yearday = tm->tm_yday;
	day = tm->tm_mday;
	if (tm->tm_year >= 70 && tm->tm_year < 138) {
		if ((long)tim == -1) {
			// This should only occur between 1/1/1970 and 12/31/1979
			tim = 24*3600*tm->tm_yday + 365*24*3600*(tm->tm_year-70);
			if (tm->tm_year < 72)
				// No leap year to consider, done
				;
			else
				if (tm->tm_year < 76)
					// Only one leap year
					tim += 24*3600;
				else 
					// two leap years
					tim += 48*3600;
		}
		printf("%lu seconds from Jan. 1, 1970\n",tim);
	}
	else
		printf("Unable to calculate seconds from Jan. 1, 1970\n");
	printf("Local Time: %s",asctime(tm));
//	printf("GMT:        %s",asctime(tm_gmt));
	printf("Date (MM/DD/YYYY) is %d/%d/%4d\n",tm->tm_mon+1,tm->tm_mday,tm->tm_year+1900);
	printf("     (DD.MM.YYYY) is %d.%d.%4d\n",tm->tm_mday,tm->tm_mon+1,tm->tm_year+1900);
	printf("     (YYYY.MM.DD) is %4d.%d.%d\n",tm->tm_year+1900,tm->tm_mon+1,tm->tm_mday);
	printf("     (YYYYDDD)    is %d\n",(long)((tm->tm_year+1900)*1000l + tm->tm_yday + 1));
//	WeekDay[day_of_week(tm->tm_year+1900, tm->tm_yday)]);
	exit(0);
}
